from tkinter import ttk
from tkinter import Tk
from tkinter import Button, Label
from .data import Message
from src import sendkeys
from typing import Callable, Dict
from src.data import log
import sv_ttk


class Gui(Tk):
    def __init__(self, parent=None) -> None:
        Tk.__init__(self, parent)
        sv_ttk.set_theme("dark")
        self.title("Uber Trader")
        # self.geometry("300x200+10+20")
        self.overrideredirect(True)  # remove border
        self.wm_withdraw()  # Hide window
        self.attributes('-topmost', True)  # always on top
        # style = ttk.Style()
        # style.configure("BW.TLabel", foreground="white", background="black")
        self.tab_control = ttk.Notebook(self, style='BW.TLabel')
        self.tab_control.pack(expand=1, fill='both')
        self.tab_control.bind("<ButtonPress-1>", self.start_move)
        self.tab_control.bind("<ButtonRelease-1>", self.stop_move)
        self.tab_control.bind("<B1-Motion>", self.do_move)
        self.tabs: Dict = dict()

    def add_tab(self, number: int, message: Message) -> None:
        if not message.communication:
            return
        self.wm_deiconify()
        user = message.communication.user
        if user not in self.tabs:
            self.tabs[user] = {'frame': ttk.Frame(self.tab_control)}
            self.tabs[user]['items'] = {}
        frame = self.tabs[user]['frame']
        if message.communication.trade is not None:
            item = message.communication.trade.item
            if item not in self.tabs[user]['items']:
                row = len(self.tabs[user]['items'])
                self.tabs[user]['items'][item] = {}
                self.tabs[user]['items'][item]['row'] = row
                self.tabs[user]['items'][item]['message'] = message
                ttk.Label(frame, text=item).grid(column=0, row=row)
                Label(frame, text=f'{message.communication.trade.amount}'
                      ' {message.communication.trade.currency}').grid(column=1, row=row)
                self.add_button(tab=frame, text='Inv', callback=self.inv_callback,
                                message=message).grid(column=2, row=row)
                self.add_button(tab=frame, text='Trade', callback=self.trade_callback,
                                message=message).grid(column=3, row=row)
                self.add_button(tab=frame, text='Ty', callback=self.ty_callback,
                                message=message).grid(column=4, row=row)
                self.add_button(tab=frame,
                                text='Wait',
                                callback=self.wait_callback,
                                message=message).grid(column=5, row=row)
                self.add_button(tab=frame, text='X', callback=self.destroy_tab, message=message).grid(column=6, row=row)
                self.tab_control.add(frame, text=user)
        else:
            log.warning(f'Trade in message "{str(message)}" is None')

    def add_button(self, tab: ttk.Frame, text: str, callback: Callable, message: Message) -> Button:
        return Button(tab, text=text, command=lambda: callback(message, tab))

    def start_move(self, event):
        self.x = event.x
        self.y = event.y

    def stop_move(self, event):
        self.x = None
        self.y = None

    def do_move(self, event):
        deltax = event.x - self.x
        deltay = event.y - self.y
        x = self.winfo_x() + deltax
        y = self.winfo_y() + deltay
        self.geometry(f"+{x}+{y}")

    def inv_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.invite(message=message)
        sendkeys.send_to_format(type='pickup', message=message)

    def ty_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.send_to_format(type='ty', message=message)
        sendkeys.kick(message=message)
        self.destroy_tab(message=message, tab=tab)

    def trade_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.trade(message=message)

    def wait_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.send_to_format(type='wait', message=message)

    def destroy_tab(self, message: Message, tab: ttk.Frame) -> None:
        if message.communication is None:
            log.error(f'Error destroying tab: no communication in message {message.message}')
            return
        if message.communication.trade is None:
            log.error(f'Error destroying tab: no trade in message {message.message}')
            return
        user = message.communication.user
        item = message.communication.trade.item
        if message.communication.user in self.tabs:
            if message.communication.trade.item in self.tabs[message.communication.user]['items']:
                # Delete the item
                row = self.tabs[message.communication.user]['items'][item]['row']
                for widget in self.tabs[user]['frame'].grid_slaves(row=row):
                    widget.destroy()
                del self.tabs[user]['items'][item]
            if len(self.tabs[user]['items']) == 0:
                # If no item from this user is left, then also delete the tab
                del self.tabs[user]
                tab.destroy()
        if not self.tabs:
            self.wm_withdraw()
