
from pywinauto.application import Application, ProcessNotFoundError
from pywinauto.findwindows import ElementNotFoundError
from src import data
from src import config


def send_text(text: str) -> None:
    app = Application()
    try:
        app.connect(path=config.conf['General']['poe_path'])
    except ProcessNotFoundError:
        data.log.warning(f'App "{config.conf["General"]["poe_window_title"]}": "{config.conf["General"]["poe_path"]}" not found')
        return
    try:
        win = app.window(title_re=config.conf['General']['poe_window_title'])
    except ElementNotFoundError:
        data.log.warning(f'Window Title "{config.conf["General"]["poe_window_title"]}" not found')
        return
    text = escape_mods(text)
    win.set_keyboard_focus()
    win.wait(wait_for='active')
    win.type_keys('{ENTER}^A{DELETE}' + text + '{ENTER}', with_spaces=True, pause=config.conf['General']['after_sendkeys_key_wait'])


def escape_mods(text: str) -> str:
    text = text.replace('%', '{%}')
    text = text.replace('+', '{+}')
    text = text.replace('^', '{^}')
    text = text.replace('(', '{(}')
    text = text.replace(')', '{)}')
    text = text.replace('~', '{~}')
    return text


def send_to(user: str, text: str) -> None:
    send_text(f'@{user} {text}')


def send_to_format(type: str, message: data.Message) -> None:
    """Send a message defined in the conf dict to a user

    Args:
        type (str): The type of message. This is defined in conf['Chat'].
        message (data.Message): Message data. Placeholders in the message type will be replaced with this data.
        conf (dict): Configution dictionary where the messages are stored.
    """
    if message.communication.trade:
        text = config.conf['Chat'][type].format(message=message.communication.message,
                                                date=message.date,
                                                channel=message.communication.channel.name,
                                                user=message.communication.user,
                                                guild=message.communication.guild,
                                                to_from=message.communication.to_from,
                                                item=message.communication.trade.item,
                                                amount=message.communication.trade.amount,
                                                currency=message.communication.trade.currency,
                                                tab=message.communication.trade.tab,
                                                row=message.communication.trade.row,
                                                col=message.communication.trade.col,
                                                league=message.communication.trade.league)
    else:
        text = config.conf['Chat'][type].format(message=message.communication.message,
                                                date=message.date,
                                                channel=message.communication.channel.name,
                                                user=message.communication.user,
                                                guild=message.communication.guild,
                                                to_from=message.communication.to_from)
    send_to(message.communication.user, text)


def invite(message: data.Message) -> None:
    send_text(f'/invite {message.communication.user}')


def join(message: data.Message) -> None:
    send_text(f'/hideout {message.communication.user}')


def return_to_ho() -> None:
    send_text('/hideout')


def kick(message: data.Message) -> None:
    send_text(f'/kick {message.communication.user}')


def leave() -> None:
    send_text('/leave')


def trade(message: data.Message) -> None:
    send_text(f'/tradewith {message.communication.user}')
